const SDLSpeaker = require('./build/Release/sdl-speaker.node');
const Channel = require('./lib/Channel');
const wrapper = require('./lib/wrapper');
const API = module.exports = {
  get state() {
    return currentState;
  }
};
const channels = {};
let stateCallback = null;
let currentState = 'close';
let checkChannelState = null;

function startCheck() {
  if (checkChannelState) { return }
  checkChannelState = setInterval(() => {
    const channelState = wrapper.speaker.getChannelsState();
    if (channelState && channelState.length) { stateCallback(channelState) }
  }, 1000)
}

function clearCheck() {
  clearInterval(checkChannelState);
  checkChannelState = null;
}

/**
 * Mix multi frame to single frame.
 * All frame must have the same length.
 * @param {Number} length - single frame length
 * @param {Array<Buffer>} frames
 * @return {Buffer}
 */
API.mix = (length, ...frames) => {
  let mixed = Buffer.alloc(length, 0);
  for (let i = 0; i < frames.length; i++) {
    let frame = frames[i];
    if (frame.length !== length) {
      throw new Error('Frame length must equal to length');
    }
    SDLSpeaker.mix(mixed, frame, frames.length);
  }
  return mixed;
}

/**
 * Init Speaker
 * @param {Object} opts
 * @param {Number} [opts.sampleRate = 16000]
 * @parma {Number} [opts.channels = 1]
 * @param {Number} [opts.samplesPerFrame = 320];
 * @param {Function} [onError]
 * @param {Function} [onState]
 */
API.init = (opts, onError, onState) => {
  opts = opts || {};
  let sampleRate = opts.sampleRate || 16000;
  let channels = opts.channels || 1;
  let samplesPerFrame = opts.samplesPerFrame || 320;
  let onErrorCb = onError || (() => {});
  stateCallback = onState || (() => {});
  wrapper.init(sampleRate, channels, samplesPerFrame, onErrorCb);
  startCheck();
  currentState = 'inited';
}

/**
 * Register a channel for the speaker. All channels is identified by name.
 * @param {String} name - Channel name.
 * @return {Channel}
 */
API.register = (name) => {
  if (channels[name]) {
    throw new Error(`Channel [${name}] is exists`);
  }
  channels[name] = new Channel(name);
  wrapper.speaker.newChannel(name);
  return channels[name];
}

/**
 * Detach a channel from the speaker.
 * @param {String} name
 */
API.detach = (name) => {
  if (channels[name]) {
    delete channels[name]
  }
  wrapper.speaker.removeChannel(name);
}

API.detachAll = () => {
  for (const channelName in channels) {
    delete channels[channelName];
    wrapper.speaker.removeChannel(channelName);
  }
}

API.version = () => {
  return SDLSpeaker.version();
}
/**
 * Start Speaker.
 */
API.start = () => {
  let res = wrapper.speaker.start();
  if (res === -1) {
    throw new Error('Current State is not Stop');
  } else if (res === -2) {
    throw new Error('SDL OpenAudio Failed');
  }
  currentState = 'playing';
  startCheck();
}

/**
 * Stop Speaker
 */
API.stop = () => {
  wrapper.speaker.stop();
  currentState = 'stop';
  clearCheck();
}

API.quit = () => {
  if (currentState !== 'stop') {
    API.stop();
  }
  API.detachAll();
  wrapper.speaker.quit();
}

/**
 * Resume Speaker
 */
API.resume = () => {
  let res = wrapper.speaker.resume();
  if (res === -1) {
    throw new Error('Current State is not Pause');
  }
  currentState = 'playing';
}

/**
 * Pause Speaker
 */
API.pause = () => {
  let res = wrapper.speaker.pause();
  if (res === -1) {
    throw new Error('Current State is not Playing');
  }
  if (currentState !== 'stop') {
    currentState = 'pause';
  }
}

/**
 * Clean all channels' buffer.
 */
API.cleanAll = () => {
  wrapper.speaker.cleanAll();
}

API.updateVolume = (volume) => {
  if (currentState !== 'stop') {
    wrapper.speaker.updateVolume(volume);
  }
}
